@@ -1,5 +1,5 @@
 use super::attributes::AttributeLocation;
-use super::{utils::*, Attribute, FromAttribute, Visibility};
+use super::{utils::*, Attribute, Visibility};
 use crate::prelude::{Delimiter, Ident, Literal, Span, TokenTree};
 use crate::{Error, Result};
 use std::iter::Peekable;
@@ -135,9 +135,9 @@ impl EnumBody {
         let stream = &mut group.stream().into_iter().peekable();
         while stream.peek().is_some() {
             let attributes = Attribute::try_take(AttributeLocation::Variant, stream)?;
-            let ident = match stream.peek() {
-                Some(TokenTree::Ident(_)) => assume_ident(stream.next()),
-                token => return Error::wrong_token(token, "ident"),
+            let ident = match super::utils::consume_ident(stream) {
+                Some(ident) => ident,
+                None => Error::wrong_token(stream.peek(), "ident")?,
             };
 
             let mut fields = Fields::Unit;
@@ -467,32 +467,6 @@ impl UnnamedField {
             None => Span::call_site(),
         }
     }
-
-    /// Check to see if the field has the given attribute. See [`FromAttribute`] for more information.
-    ///
-    /// **note**: Will immediately return `Err(_)` on the first error `T` returns.
-    pub fn has_attribute<T: FromAttribute + PartialEq<T>>(&self, attrib: T) -> Result<bool> {
-        for attribute in self.attributes.iter() {
-            if let Some(attribute) = T::parse(&attribute.tokens)? {
-                if attribute == attrib {
-                    return Ok(true);
-                }
-            }
-        }
-        Ok(false)
-    }
-
-    /// Returns the first attribute that returns `Some(Self)`. See [`FromAttribute`] for more information.
-    ///
-    /// **note**: Will immediately return `Err(_)` on the first error `T` returns.
-    pub fn get_attribute<T: FromAttribute>(&self) -> Result<Option<T>> {
-        for attribute in self.attributes.iter() {
-            if let Some(attribute) = T::parse(&attribute.tokens)? {
-                return Ok(Some(attribute));
-            }
-        }
-        Ok(None)
-    }
 }
 
 /// Reference to an enum variant's field. Either by index or by ident.
@@ -554,38 +528,12 @@ impl<'a> IdentOrIndex<'a> {
         }
     }
 
-    /// Check to see if the field has the given attribute. See [`FromAttribute`] for more information.
-    ///
-    /// **note**: Will immediately return `Err(_)` on the first error `T` returns.
-    pub fn has_attribute<T: FromAttribute + PartialEq<T>>(&self, attrib: T) -> Result<bool> {
-        let attributes = match self {
+    /// Returns the attributes of this field.
+    pub fn attributes(&self) -> &Vec<Attribute> {
+        match self {
             Self::Ident { attributes, .. } => attributes,
             Self::Index { attributes, .. } => attributes,
-        };
-        for attribute in attributes.iter() {
-            if let Some(attribute) = T::parse(&attribute.tokens)? {
-                if attribute == attrib {
-                    return Ok(true);
-                }
-            }
         }
-        Ok(false)
-    }
-
-    /// Returns the first attribute that returns `Some(Self)`. See [`FromAttribute`] for more information.
-    ///
-    /// **note**: Will immediately return `Err(_)` on the first error `T` returns.
-    pub fn get_attribute<T: FromAttribute>(&self) -> Result<Option<T>> {
-        let attributes = match self {
-            Self::Ident { attributes, .. } => attributes,
-            Self::Index { attributes, .. } => attributes,
-        };
-        for attribute in attributes.iter() {
-            if let Some(attribute) = T::parse(&attribute.tokens)? {
-                return Ok(Some(attribute));
-            }
-        }
-        Ok(None)
     }
 }
 
